<template>
  <gree-view>
    <gree-page class="page-timetaskset" :class="{ empty: !timerTasks.length && !isLoading }">
      <gree-header :left-options="{ showBack: false }" :right-options="{ showMore: false }">
        <span v-text="titleInfo" />
        <span
          v-show="delType"
          v-text="$language('timer.btn_cancel')"
          slot="left"
          @click="
            delType = false;
            timerSelcetList = [];
          "
        />
        <i v-show="!delType" class="iconfont iconfont-fanhui" slot="overwrite-left" @click="goBack" />
        <gree-check v-if="delType" v-model="timerSelectAllFlag" :label="$language('timer.allSelect')" slot="right" />
        <span v-if="!delType && timerTasks.length" v-text="$language('timer.management')" slot="right" @click="delType = true" />
      </gree-header>
      <LoadingView :loaded="!isLoading" />
      <div class="timetaskset-manage" v-show="!isLoading">
        <p class="emptyText" v-if="!timerTasks.length" v-text="$language('timer.no_data')" />
        <!-- 定时列表 -->
        <div v-if="timerTasks.length" class="timetaskset-manage-list">
          <!-- 列表内容 -->
          <div
            class="timetaskset-manage-list-content"
            v-for="(timerTask, timerTaskIndex) in timerTasks"
            :key="timerTaskIndex"
            @click="selectOne(timerTaskIndex)"
          >
            <!-- 左边内容 -->
            <div class="timetaskset-manage-list-content-left">
              <p>
                <!-- 时间 -->
                <span v-text="parseTime(timerTask.timer.Etime, timerTask.timer.timeZone)" />
                <!-- 动作 -->
                <span v-html="`&nbsp; ${getPowType(timerTask.task.cmd) ? $language('timer.turn_off') : $language('timer.turn_on')}`" />
              </p>
              <p class="timetaskset-manage-list-content-left-footer" v-text="parseDate(timerTask.timer.weeks) || ''" />
            </div>
            <!-- 右边内容 -->
            <div class="timetaskset-manage-list-content-right" @click.stop>
              <gree-check v-if="delType" class="card-item-right-check" v-model="timerSelcetList[timerTaskIndex]" />
              <gree-switch
                v-else
                class="card-item-right-switch"
                inactive-color="#c5c5c5"
                :size="switchSize"
                v-model="listSwitchType[timerTaskIndex]"
                @change="changeTask(timerTaskIndex)"
              />
            </div>
          </div>
        </div>
      </div>
      <!-- 底部按钮 -->
      <gree-button
        class="timetaskset-bottom-button"
        :class="{ 'gray-btn': delAble && delType }"
        type="primary"
        size="large"
        :icon="iconType"
        :disabled="isLoading"
        round
        :text="buttonText"
        @click="onClick"
      />
      <TimerPicker @confirm="getPickerData" :timer-task-time="timerTaskTime" :timer-task-weeks="timerTaskWeeks" :timer-i-d="timerID" />
    </gree-page>
  </gree-view>
</template>

<script>
import { Header, Switch, Check, button, Icon, ActivityIndicator, Dialog, Toast } from 'gree-ui';
import { setCloudTimer, modifyCloudTimer, deleteCloudTimer, getCloudTimerByMac } from '@PluginInterface';
import { mapState, mapMutations, mapGetters } from 'vuex';
import { analysisTimer } from '@/utils/encryptionFun';
import LoadingView from '@components/LoadingView';
import TimerPicker from '@/components/Popup/TimerPicker';

export default {
  components: {
    [Header.name]: Header,
    [Switch.name]: Switch,
    [Check.name]: Check,
    [button.name]: button,
    [Icon.name]: Icon,
    [ActivityIndicator.name]: ActivityIndicator,
    [TimerPicker.name]: TimerPicker,
    LoadingView
  },
  data() {
    return {
      isLoading: true, //是否显示loading
      timerSelcetList: [],
      timerSelectAllFlag: false,
      timerTasks: [],
      listSwitchType: [],
      delType: false, // 删除状态
      timerTaskTime: {},
      timerTaskWeeks: [],
      timerID: 0,
      delAble: false,
      isShowLoading: false
    };
  },
  mounted() {
    // 查一包云定时
    this.localGetCloudTimer();
  },
  destroyed() {
    this.SetPopup({ timerPicker: false });
  },
  computed: {
    ...mapState('control', {
      mac: state => state.mac,
      mainMac: state => state.mainMac
    }),
    ...mapGetters(['modCurrentStatusName', 'spModCurrentStatusName']),
    isAllSelect() {
      return this.timerTasks.every((v, i) => {
        return this.timerSelcetList[i];
      });
    },
    isAllBlur() {
      return !this.timerTasks.some((v, i) => {
        return this.timerSelcetList[i];
      });
    },
    switchSize() {
      return Math.floor((window['lib'].flexible.rem / 108) * 88);
    },
    checkTimeIndex() {
      const result = [];
      const timerSelcetArr = Object.values(this.timerSelcetList);
      timerSelcetArr.map(index => {
        index && result.push(index);
      });
      return result.length;
    },
    titleInfo() {
      return !this.delType ? window.myvm.$i18n.t('timer.functionPage_title') : window.myvm.$i18n.t('timer.select_number') + `（${this.checkTimeIndex}）`;
    },
    buttonText() {
      return !this.delType ? window.myvm.$i18n.t('timer.add') : window.myvm.$i18n.t('timer.btn_delet');
    },
    iconType() {
      return !this.delType ? 'add' : 'delete';
    }
  },
  watch: {
    'timerTasks.length': {
      handler() {
        // 更新高度
        this.updateBoxHeight();
        this.selectAll();
      },
      immediate: true
    },
    timerSelectAllFlag(newVal) {
      newVal !== 0 && this.selectAll(newVal);
    },
    isAllSelect(newVal) {
      if (newVal) {
        this.timerSelectAllFlag = true;
      } else {
        this.timerSelectAllFlag = 0;
      }
    },
    timerSelcetList() {
      this.timerSelcetList.includes(true) ? (this.delAble = false) : (this.delAble = true);
    },
    isAllBlur(newVal) {
      newVal && (this.timerSelectAllFlag = false);
    }
  },
  methods: {
    ...mapMutations('control', {
      setDataObject: 'SET_DATA_OBJECT',
      setState: 'CONTROL_SET_STATE',
      SetPopup: 'SET_POPUP'
    }),
    localGetCloudTimer() {
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac
      return getCloudTimerByMac(sendMac).then(res => {
        const result = JSON.parse(res);
        if (result.r === 200) {
          this.isLoading = false;
          this.timerTasks = analysisTimer(result.timerTasks).reverse();
          // 更新开关状态
          const hasTaskOn = this.updateSwitchType();
          this.setDataObject({ TimerSet: hasTaskOn });
          // 更新定时小图标文案
          this.getCloseTimer();
        }
      });
    },
    // 更新开关状态
    updateSwitchType() {
      let hasTaskOn = false;
      this.listSwitchType = this.timerTasks
        ? this.timerTasks.map(timerTask => {
            const type = Boolean(timerTask.timer.status);
            hasTaskOn = hasTaskOn || type;
            return type;
          })
        : [];
      return hasTaskOn;
    },
    // 更新高度
    updateBoxHeight() {
      this.$emit('updateHeight');
    },
    // 获取开关状态 1关 0开
    getPowType(cmd) {
      const { opt, p } = cmd;
      const powIndex = opt.findIndex(v => v === 'Pow');
      const powType = powIndex === -1 ? 1 : Number(!p[powIndex]);
      return powType;
    },
    // 时间解析
    parseTime(stringTime, timeZone) {
      if (!stringTime) return;
      const timeList = stringTime.split(':');
      let hour = Number(timeList[0]) + timeZone;
      if (hour >= 24) hour -= 24;
      if (hour < 10) hour = `0${hour}`;
      return `${hour}:${timeList[1]}  `;
    },
    parseTimeToMap(timer, task) {
      const { Etime, weeks, timeZone, timerID } = timer;
      const targetTime = this.parseTime(Etime, timeZone);
      const type = this.getPowType(task.cmd);
      const [hour, minus] = targetTime.split(':').map(Number);
      return {
        hour,
        minus,
        type,
        weeks,
        timerID
      };
    },
    // 解析日期
    parseDate(list) {
      if (!list) return;
      if (list.length === 7) return this.$language('timer.every');
      if (list.length === 0 || list[0] === '') return this.$language('timer.once');
      const valueObj = this.$language('timer.Mon', 'timer.Tue', 'timer.Wed', 'timer.Thu', 'timer.Fir', 'timer.Sat', 'timer.Sun');
      let returnValue = '';
      list.forEach(element => {
        if (element === '') return;
        returnValue += `${valueObj[Number(element) - 1]} `;
      });
      return returnValue.substring(0, returnValue.length - 1);
    },
    async showTimerSetPopup(isCheck = false) {
      if (isCheck) {
        await this.localGetCloudTimer();
        if (this.timerTasks.length >= 10) {
          this.$toast(`${this.$language('timer.toast')}`);
          return;
        }
      }
      this.SetPopup({ timerPicker: true });
    },
    changeTask(index) {
      this.showLoading();
      const { task, timer } = this.timerTasks[index];
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac

      const timerDic = {
        timeType: timer.timeType,
        weeks: timer.weeks,
        Etime: timer.Etime,
        timeZone: timer.timeZone,
        status: Number(!timer.status)
      };
      const taskDic = {
        mac: sendMac,
        subId: mainMac.length ? mac : '',
        cmd: JSON.stringify(task.cmd),
        remark: task.remark,
        dat: task.dat
      };
      modifyCloudTimer(timerDic, taskDic, timer.timerID).then(() => {
        // 保存成功主动查一次
        this.localGetCloudTimer();
        this.hideLoading();
      });
    },
    //删除定时弹框
    onDelete() {
      Dialog.confirm({
        message: this.$language('timer.Dialog_content'),
        confirmButtonText: this.$language('timer.Dialog_confirmText'),
        cancelButtonText: this.$language('timer.Dialog_cancelText'),
        confirmButtonColor: '#ff0202'
      })
        .then(() => {
          this.delTimers();
        })
        .catch(() => {
          return;
        });
    },
    delTimers() {
      this.delType = false;
      this.timerSelectAllFlag = false;
      const timerIDs = [];
      this.timerSelcetList.forEach((value, index) => {
        try {
          value && timerIDs.push(this.timerTasks[index].timer.timerID);
        } catch (e) {
          e;
        }
      });
      deleteCloudTimer(timerIDs).then(() => {
        this.$toast(`${this.$language('timer.Delete_success')}`);
        this.localGetCloudTimer();

        // 删除成功主动查一次
      });
    },
    setPickerPropData(index = -1) {
      if (index === -1) {
        //新增定时
        const now = new Date();
        this.timerTaskTime = { hour: now.getHours(), minus: now.getMinutes(), type: 0 };
        this.timerTaskWeeks = [];
        this.timerID = 0;
      } else {
        //修改定时
        const { timer, task } = this.timerTasks[index];
        const { hour, minus, type, weeks, timerID } = this.parseTimeToMap(timer, task);

        this.timerTaskTime = {
          hour,
          minus,
          type
        };
        this.timerTaskWeeks = weeks;
        this.timerID = timerID;
      }
    },
    selectOne(index) {
      if (this.delType) {
        this.$set(this.timerSelcetList, index, !this.timerSelcetList[index]);
      } else {
        // 设置picker数据
        this.setPickerPropData(index);
        // 显示picker弹窗
        this.showTimerSetPopup();
      }
    },
    selectAll(type) {
      const setTimerAll = type => {
        this.timerSelcetList = this.timerTasks.map(() => type);
      };
      if (type === undefined) {
        this.isAllSelect && setTimerAll(true);
        this.isAllBlur && setTimerAll(false);
      } else {
        setTimerAll(type);
      }
    },
    async getPickerData(val) {
      await this.localGetCloudTimer();
      const { mac, mainMac } = this;
      const sendMac = mainMac.length ? `${mac}@${mainMac}` : mac; // 查询包需要传入主mac及子mac

      const [pickerData, weekList, timerID] = val;

      const hour = (((Number(pickerData[0]) + (24 - 8)) % 24) + '').padStart(2, '0');
      const minus = (pickerData[1] + '').padStart(2, '0');
      const Etime = `${hour}:${minus}:00`;
      const Pow = pickerData[2].includes(this.$t('timer.turn_on')) ? 1 : 0;
      const p = [Pow, 0, 0, 0, 0];
      const opt = ['Pow', 'Health', 'SlpMod', 'SwhSlp', 'StHt'];

      const weeks = [];
      weekList.forEach(week => {
        week.select && weeks.push(week.index);
      });
      const timerDic = {
        timeType: weeks.length ? 2 : 1,
        weeks,
        Etime,
        timeZone: 8,
        status: 1
      };
      const taskDic = {
        mac: sendMac,
        subId: mainMac.length ? mac : '', //多联机subId需要传入子mac
        cmd: JSON.stringify({ t: 'cmd', sub: mac, opt, p }),
        remark: 'i am lazy, no remark',
        dat: p.map(String)
      };

      if (timerID && this.timerTasks.find(timerTask => timerTask.timer.timerID === timerID)) {
        modifyCloudTimer(timerDic, taskDic, timerID).then(() => {
          this.$toast(`${this.$language('timer.toast1')}`);
          // 保存成功主动查一次
          this.localGetCloudTimer();
        });
      } else {
        if (this.timerTasks.length >= 10) {
          this.$toast(`${this.$language('timer.toast')}`);
        } else {
          console.log('setCloudTimer--------------参数：', JSON.stringify(timerDic), JSON.stringify(taskDic));
          setCloudTimer(timerDic, taskDic).then(() => {
            this.$toast(`${this.$language('timer.toast1')}`);
            // 保存成功主动查一次
            this.localGetCloudTimer();
          });
        }
      }
    },
    // 最近的一个定时
    getCloseTimer() {
      const timerTaskList = this.timerTasks.filter(timerTask => timerTask.timer.status);
      const date = new Date();
      const timerList = [];
      const targetTimer = timerTaskList
        .map(timerTask => {
          let diffTime;
          const { timer, task } = timerTask;
          const { hour, minus, type, weeks, timerID } = this.parseTimeToMap(timer, task);
          timerList.push({ hour, minus, type, timerID });
          if (weeks.length && weeks[0] !== '') {
            // 轮询选择的日期，返回最近的结果
            diffTime = weeks
              .map(day => {
                return date.dateDiff({
                  hour,
                  minus,
                  day: day % 7
                });
              })
              .sort((a, b) => a.time - b.time)[0];
          } else {
            // 返回当前时间与选择时间的差值
            diffTime = date.dateDiff({
              hour,
              minus
            });
          }
          return { timerID, ...diffTime };
        })
        .sort((a, b) => a.time - b.time)[0];
      const result = timerList && timerList.find(timer => timer.timerID === targetTimer.timerID);
      if (!result) {
        this.setState({ closeTimer: '' });
      } else {
        const timeString = `${this.padZero(result.hour)}:${this.padZero(result.minus)}`;
        this.setState({ closeTimer: `${timeString}${result.type ? '关机' : '开机'}` });
      }
    },

    //数字补零
    padZero(num, targetLength = 2) {
      let str = num + '';
      while (str.length < targetLength) {
        str = '0' + str;
      }
      return str;
    },
    goBack() {
      // this.isLoading = true;
      this.$router.push('Home');
    },
    onClick() {
      if (!this.delType) {
        this.setPickerPropData();
        this.showTimerSetPopup(true);
      } else {
        this.onDelete();
      }
    },
    showLoading() {
      Toast.loading('');
    },
    hideLoading() {
      !this.isShowLoading && Toast.clear();
    }
  }
};
</script>

<style lang="scss">
.page-timetaskset {
  width: 100%;
  height: 100%;
  .gree-header {
    background-color: white;
    background-image: none;
    .gree-check {
      .gree-check-icon {
        position: relative;
        color: #383838;
        .gree-icon {
          color: #71acf1;
        }
      }
    }
  }
  .timetaskset-manage {
    width: 100%;
    display: flex;
    justify-content: center;
    align-items: flex-start;
    &-list {
      height: 100%;
      width: 100%;
      &-content {
        height: 150px;
        background-image: linear-gradient(0deg, rgba(207, 204, 204, 0.3), rgba(207, 204, 204, 0.3));
        background-size: 100% 1px;
        background-repeat: no-repeat;
        background-position: top;
        background-color: white;
        display: flex;
        justify-content: space-between;
        align-items: center;
        font-size: 44px;
        &-left {
          padding-left: 52px;
          color: #404657;
          &-footer {
            padding-top: 8px;
            font-size: 36px;
            color: #989898;
          }
        }
        &-right {
          padding-right: 60px;
          .card-item-right-check {
            .gree-check-icon {
              color: #c5cad5;
              .gree-icon {
                color: #71acf1;
              }
            }
          }
        }
      }
    }
  }
  .timetaskset-bottom-button {
    height: 114px;
    width: 879px;
    line-height: 114px;
    margin-left: 100px;
    position: absolute;
    bottom: 85px;
  }
  .gray-btn {
    opacity: 0.3;
    pointer-events: none;
  }
}
.empty {
  position: absolute;
  background-image: url('../../assets/img/nodata.png');
  background-repeat: no-repeat;
  background-position: 55% 40%;
  background-size: 65%;
  background-color: white;
  .emptyText {
    font-size: 46px;
    color: #989898;
    padding-top: 50vh;
  }
}
</style>
